@@ -4,15 +4,14 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module     : Control.Selective
--- Copyright  : (c) Andrey Mokhov 2018-2023
+-- Copyright  : (c) Andrey Mokhov 2018-2024
 -- License    : MIT (see the file LICENSE)
 -- Maintainer : andrey.mokhov@gmail.com
 -- Stability  : experimental
 --
 -- This is a library for /selective applicative functors/, or just
 -- /selective functors/ for short, an abstraction between applicative functors
--- and monads, introduced in this paper:
--- https://www.staff.ncl.ac.uk/andrey.mokhov/selective-functors.pdf.
+-- and monads, introduced in this paper: https://dl.acm.org/doi/10.1145/3341694.
 --
 -----------------------------------------------------------------------------
 module Control.Selective (
@@ -297,7 +296,14 @@ whenS x y = select (bool (Right ()) (Left ()) <$> x) (const <$> y)
 
 -- | A lifted version of 'Data.Maybe.fromMaybe'.
 fromMaybeS :: Selective f => f a -> f (Maybe a) -> f a
-fromMaybeS x mx = select (maybe (Left ()) Right <$> mx) (const <$> x)
+fromMaybeS fNothing fMaybe =
+    select (toEither <$> fMaybe) (toUnitFunction <$> fNothing)
+  where
+    toEither :: Maybe a -> Either () a
+    toEither Nothing  = Left ()
+    toEither (Just a) = Right a
+    toUnitFunction :: a -> () -> a
+    toUnitFunction x () = x
 
 -- | Return the first @Right@ value. If both are @Left@'s, accumulate errors.
 orElse :: (Selective f, Semigroup e) => f (Either e a) -> f (Either e a) -> f (Either e a)
