@@ -1,6 +1,6 @@
-{-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE DeriveFunctor, EmptyCase, FlexibleInstances, GADTs, RankNTypes #-}
 {-# LANGUAGE MultiParamTypeClasses, ScopedTypeVariables, TupleSections #-}
+{-# OPTIONS_GHC -fno-warn-unused-imports #-}
 module Sketch where
 
 import Control.Arrow hiding (first, second)
@@ -10,9 +10,6 @@ import Control.Selective
 import Data.Bifunctor
 import Data.Bool
 import Data.Function
-import Data.Functor
-import Data.Functor.Identity
-import Data.Functor.Const
 import Data.Semigroup (Semigroup (..))
 import Data.Void
 
@@ -20,7 +17,10 @@ import qualified Control.Arrow    as A
 import qualified Control.Category as C
 
 -- This file contains various examples and proof sketches and we keep it here to
--- make sure it still compiles.
+-- make sure it still compiles. We ignore HLINT suggestions because they often
+-- get in the way of readable "proofs" that use equational reasoning.
+
+{-# ANN module "HLint: ignore" #-}
 
 ------------------------------- Various examples -------------------------------
 
@@ -565,7 +565,7 @@ data Result a = Done a | Blocked BlockedRequests (Haxl a) deriving Functor
 newtype Haxl a = Haxl { runHaxl :: IO (Result a) } deriving Functor
 
 instance Applicative Haxl where
-    pure = return
+    pure = Haxl . return . Done
 
     Haxl iof <*> Haxl iox = Haxl $ do
         rf <- iof
@@ -586,130 +586,9 @@ instance Selective Haxl where
             (Blocked bx x  , Blocked bf f) -> Blocked (bx <> bf) (select x f) -- speculative
                                                                               -- execution
 instance Monad Haxl where
-    return = Haxl . return . Done
+    return = pure
 
     Haxl iox >>= f = Haxl $ do
         rx <- iox
         case rx of Done       x -> runHaxl (f x) -- dynamic dependency on runtime value 'x'
                    Blocked bx x -> return (Blocked bx (x >>= f))
-
--- right' :: Choice p => p a b -> p (Either c a) (Either c b)
--- right' :: ... => (a -> f b) -> Either x a -> f (Either x b)
-
-data P s t a b = P { match' :: s -> Either a t, build' :: b -> t }
-
-fromP :: P s t a b -> Prism s t a b
-fromP (P match build) f s = case match s of
-    Left  a -> build <$> f a
-    Right t -> pure t
-
-
--- Choice p, Applicative f) => p a (f b) -> p s (f t)
-
--- (a -> f b) -> (Either a x -> f (Either b x))
-
-type Lens      s t a b = forall f. Functor f     => (a -> f b) -> s -> f t
-type Traversal s t a b = forall f. Applicative f => (a -> f b) -> s -> f t
-type Prism     s t a b = forall f. Selective f   => (a -> f b) -> s -> f t
-
-_fst :: Lens (a, x) (b, x) a b
-_fst f (a, x) = f a <&> (,x)
-
-_snd :: Lens (x, a) (x, b) a b
-_snd f (x, a) = (x,) <$> f a
-
-_Left :: Prism (Either a x) (Either b x) a b
-_Left f = \case Left  a -> Left <$> f a
-                Right x -> pure (Right x)
-
-view :: Lens s t a b -> s -> a
-view lens s = getConst $ lens (\a -> Const a) s
-
-update :: Lens s t a b -> b -> s -> t
-update lens b s = runIdentity $ lens (\_a -> Identity b) s
-
-match :: Prism s t a b -> s -> Either a t
-match prism s = prism Left s
-
--- (a -> f s b) -> s -> f s t
-build :: Prism s t a b -> b -> t
-build prism b = either absurd id $ prism (\_a -> Right b) undefined
-
-
--- x <*? (y <*? z)
-
--- 1      0     0
--- 1      1     0
--- 1      1     1
-
--- (x <*? y) <*? z
-
--- 1      0      0
--- 1      1      0
--- 1      0      1
--- 1      1      1
-
-
--- data Evaluation e a = Unknown e | Known a | Wrapped (Evaluation e a)
---     deriving (Functor, Show)
-
--- type E e a = Identity
-
--- instance Semigroup e => Applicative (Evaluation e) where
---     pure = Known
---     Unknown e1 <*> Unknown e2 = Unknown (e1 <> e2)
---     Unknown e1 <*> Known _    = Unknown e1
---     Known _    <*> Unknown e2 = Unknown e2
---     Known f    <*> Known a    = Known (f a)
-
--- instance Semigroup e => Selective (Evaluation e) where
---     select (Known (Right b)) _            = Known b
---     select (Known (Left  a)) f            = ($a) <$> f
---     select (Unknown e1     ) (Known   _ ) = Unknown e1
---     select (Unknown e1     ) (Unknown e2) = Unknown (e1 <> e2)
-
-
-data C f g a where
-    C :: f x -> g y -> (x -> Either (y -> a) a) -> C f g a
-
-instance Functor (C f g) where
-    fmap f (C p q k) = C p q (fmap (bimap (f.) f) k)
-
-instance (Applicative f, Applicative g) => Applicative (C f g) where
-    pure a = C (pure ()) (pure ()) (const (Left (const a)))
-    C p1 q1 k1 <*> C p2 q2 k2 = C ((,) <$> p1 <*> p2) ((,) <$> q1 <*> q2) $
-        \(x1, x2) -> Left (\(y1, y2) ->
-            (either ($y1) id (k1 x1)) (either ($y2) id (k2 x2)))
-
--- class Applicative f => SelectiveQ f where
---     select :: Enumerable t => f (g a) -> (t ~> f :*: u) -> f (Sigma u)
-
--- instance (Selective f, Selective g) => Selective (f :*: g) where
-
--- instance Selective (Maybe :*: Maybe) where
-
--- instance Selective Maybe where
---     match sigma pi = sigma >>= \s -> pureSelect s pi
-
-
-
--- data Value a b = Likely a | Unlikely b
--- Some kind of "Sigma arrows"
--- Sigma t -> (Match t (Sigma s)) -> Sigma s
--- Sigma t -> (t ~> Sigma s) -> Sigma s
-
--- branchU :: forall f a b c. SelectiveU f => f (Either a b) -> f (a -> c) -> f (b -> c) -> f c
--- branchU x f g = selectU (toAB <$> x) h
---   where
---     -- toAB :: Either a b -> (AB a b x, x)
---     toAB (Left  a) = (A, a)
---     toAB (Right b) = (B, b)
---     -- h :: AB a b x -> f (x -> c)
---     h A = f
---     h B = g
-
--- applyS :: forall f a b. SelectiveS f => f a -> f (a -> b) -> f b
--- applyS x f = sigmaSelect (Sigma Refl <$> x) h
---   where
---     h :: forall x. Refl a x -> f (x -> b)
---     h Refl = f
